@@ -31,6 +31,8 @@ extern "C" {
 #endif
 
 bool l_main_init(void);
+int l_main_prepare(void);
+void l_main_iterate(int timeout);
 int l_main_run(void);
 bool l_main_exit(void);
 
@@ -40,6 +42,8 @@ typedef void (*l_main_signal_cb_t) (uint32_t signo, void *user_data);
 
 int l_main_run_with_signal(l_main_signal_cb_t callback, void *user_data);
 
+int l_main_get_epoll_fd();
+
 #ifdef __cplusplus
 }
 #endif
